from __future__ import annotations

import base64
import hashlib
import hmac
import json
import urllib.parse
import uuid
from dataclasses import asdict, dataclass
from enum import Enum
from typing import Literal

from .config import OmniConfig, OmniConfigError
from .utils import compact_json_dump


@dataclass
class DashboardEmbedUrl:
    base_url: str
    contentPath: str
    externalId: str
    name: str
    nonce: str
    accessBoost: str | None = None
    connectionRoles: str | None = None
    customTheme: str | None = None
    customThemeId: str | None = None
    email: str | None = None
    entity: str | None = None
    entityFolderContentRole: str | None = None
    entityFolderGroupContentRole: str | None = None
    entityFolderLabel: str | None = None
    entityGroupLabel: str | None = None
    filterSearchParam: str | None = None
    groups: str | None = None
    linkAccess: str | None = None
    mode: str | None = None
    modelRoles: str | None = None
    prefersDark: str | None = None
    preserveEntityFolderContentRole: str | None = None
    theme: str | None = None
    uiSettings: str | None = None
    userAttributes: str | None = None
    signature: str | None = None

    def __str__(self) -> str:
        """String representation renders the complete URL for the embedded dashboard."""
        params = asdict(self)
        del params["base_url"]
        empty_keys = [key for key, value in params.items() if value is None]
        for key in empty_keys:
            del params[key]
        return f"{self.base_url}?{urllib.parse.urlencode(params)}"


class OmniDashboardEmbedder:
    """Factory class for building and signing dashboard embedding URLs.

    Args:
        organization_name: organization_name: Omni organization name. OMNI_ORGANIZATION_NAME environment variable will
            be used as a fallback.
        embed_secret: Omni embed secret. OMNI_EMBED_SECRET environment variable will be used as a fallback.
        vanity_domain: Vanity domain configured with Omni. Should not be fully qualified. OMNI_VANITY_DOMAIN
            environment variable will be used as a fallback.

    Attributes:
        embed_login_url: Base url of embedded dashboard urls.
        embed_secret: Omni embed secret.
    """

    class AccessMode(Enum):
        """AccessMode options

        Attributes:
            application: APPLICATION
            single_content: SINGLE_CONTENT
        """

        application = "APPLICATION"
        single_content = "SINGLE_CONTENT"

    class ContentRole(Enum):
        """ContentRole options

        Attributes:
            viewer: VIEWER
            editor: EDITOR
            manager: MANAGER
            no_access: NO_ACCESS
        """

        viewer = "VIEWER"
        editor = "EDITOR"
        manager = "MANAGER"
        no_access = "NO_ACCESS"

    class PrefersDark(Enum):
        """PrefersDark options

        Attributes:
            yes: true
            no: false
            system: system
        """

        yes = "true"
        no = "false"
        system = "system"

    class Theme(Enum):
        """Theme options

        Attributes:
            dawn: dawn
            vibes: vibes
            breeze: breeze
            blank: blank
        """

        dawn = "dawn"
        vibes = "vibes"
        breeze = "breeze"
        blank = "blank"

    def __init__(
        self,
        organization_name: str | None = None,
        embed_secret: str | None = None,
        vanity_domain: str | None = None,
    ):
        omni_config = OmniConfig(
            required_attrs=["embed_secret"],
            organization_name=organization_name,
            embed_secret=embed_secret,
            vanity_domain=vanity_domain,
        )
        if not omni_config.vanity_domain and not omni_config.organization_name:
            raise OmniConfigError(
                "You must pass the vanity_domain or organization_name arguments OR "
                "set the OMNI_ORGANIZATION_NAME or OMNI_VANITY_DOMAIN environment variables."
            )
        embed_host = (
            omni_config.vanity_domain
            or f"{omni_config.organization_name}.embed-omniapp.co"
        )
        self.embed_login_url = f"https://{embed_host}/embed/login"

        # Required to appease mypy. If embed_secret is missing an OmniConfigError will have already been raised by the OmniConfig class.
        assert omni_config.embed_secret

        self.embed_secret = omni_config.embed_secret

    def build_url(
        self,
        content_path: str,
        external_id: str,
        name: str,
        access_boost: bool | None = None,
        connection_roles: dict | None = None,
        custom_theme: dict | None = None,
        custom_theme_id: str | None = None,
        email: str | None = None,
        entity: str | None = None,
        entity_folder_content_role: ContentRole | None = None,
        entity_folder_group_content_role: ContentRole | None = None,
        entity_folder_label: str | None = None,
        entity_group_label: str | None = None,
        filter_search_params: str | dict | None = None,
        groups: list[str] | None = None,
        link_access: bool | list[str] | None = None,
        mode: AccessMode | None = None,
        model_roles: dict | None = None,
        prefers_dark: PrefersDark | None = None,
        preserve_entity_folder_content_role: bool | None = None,
        theme: Theme | None = None,
        ui_settings: dict | None = None,
        user_attributes: dict | None = None,
    ) -> str:
        """
        Builds a signed dashboard embedding URL. For more information on the options see the Omni Docs:
        https://docs.omni.co/embed/setup/url-parameters

        Args:
            access_boost (bool, optional): Enables AccessBoost for the embedded dashboard.
            connection_roles (dict, optional): Level of access for all models in a connection.
            content_path (str): Path pointing to the dashboard you wish to embed.
            custom_theme (dict, optional): Custom theme properties for styling embedded dashboards.
            custom_theme_id (str, optional): Theme ID from your Omni instance to stylize embedded dashboards.
            email (str, optional): Email for entity users when sharing content or sending deliveries.
            entity (str, optional): User group identifier to associate the embed user with a larger group.
            entity_folder_content_role (str, optional): Content role for the embed user's shared entity folder.
            entity_folder_group_content_role (str, optional): Content role for the embed entity group shared folder.
            entity_folder_label (str, optional): Label for the embed user's associated entity folder.
            entity_group_label (str, optional): Label for the embed user's associated entity group.
            external_id (str): Unique ID for the embed user.
            filter_search_params (str | dict, optional): Filters to apply for the embedded content.
            groups (list[str], optional): Associate embed user with existing user groups in your Omni instance.
            link_access (bool | list[str], optional): Controls which Omni dashboards can be linked to from the embedded dashboard.
            mode (AccessMode, optional): Type of access users will have to Omni in the iframe.
            model_roles (dict, optional): Level of access for individual models in a connection.
            name (str): Name for the embed user's name property.
            prefers_dark (PrefersDark, optional): Light or dark mode appearance.
            preserve_entity_folder_content_role (bool, optional): Retains the embed user's existing entity folder content role.
            theme (Theme, optional): Built-in Omni application theme.
            ui_settings (dict, optional): General settings of the application in embed.
            user_attributes (dict, optional): User attributes to apply to the embed user.

        Returns:
            str: Signed dashboard embedding URL.
        """

        # Preprocess some values before passing to URL object.
        if link_access is True:
            _link_access = "__omni_link_access_open"
        elif isinstance(link_access, list):
            _link_access = ",".join(link_access)
        elif not link_access:
            _link_access = None
        else:
            raise ValueError(
                "link_access must be a list of dashboard IDs or True to allow links to all dashboards."
            )

        # Convert empty dicts and strings to None.
        filter_search_params = filter_search_params or None
        if isinstance(filter_search_params, dict):
            filter_search_params = urllib.parse.urlencode(
                filter_search_params, doseq=True
            )

        url = DashboardEmbedUrl(
            base_url=self.embed_login_url,
            contentPath=content_path,
            externalId=external_id,
            name=name,
            accessBoost="true" if access_boost else None,
            connectionRoles=(
                compact_json_dump(connection_roles) if connection_roles else None
            ),
            customTheme=compact_json_dump(custom_theme) if custom_theme else None,
            customThemeId=custom_theme_id,
            email=email,
            entity=entity,
            entityFolderContentRole=(
                entity_folder_content_role.value if entity_folder_content_role else None
            ),
            entityFolderGroupContentRole=(
                entity_folder_group_content_role.value
                if entity_folder_group_content_role
                else None
            ),
            entityFolderLabel=entity_folder_label,
            entityGroupLabel=entity_group_label,
            filterSearchParam=filter_search_params,
            groups=compact_json_dump(groups) if groups else None,
            linkAccess=_link_access,
            mode=mode.value if mode else None,
            modelRoles=compact_json_dump(model_roles) if model_roles else None,
            prefersDark=prefers_dark.value if prefers_dark else None,
            preserveEntityFolderContentRole=(
                "true" if preserve_entity_folder_content_role else None
            ),
            theme=theme.value if theme else None,
            uiSettings=compact_json_dump(ui_settings) if ui_settings else None,
            userAttributes=(
                compact_json_dump(user_attributes) if user_attributes else None
            ),
            nonce=uuid.uuid4().hex,
        )

        self._sign_url(url)
        return str(url)

    def _sign_url(self, url: DashboardEmbedUrl) -> None:
        """Creates a signature and adds it to the URL object."""

        # IMPORTANT: These must be in the correct order as documented here
        # https://docs.omni.co/embed/setup/standard-sso#manual-generation

        blob_items = [
            url.base_url,
            url.contentPath,
            url.externalId,
            url.name,
            url.nonce,
            url.accessBoost,
            url.connectionRoles,
            url.customTheme,
            url.customThemeId,
            url.email,
            url.entity,
            url.entityFolderContentRole,
            url.entityFolderGroupContentRole,
            url.entityFolderLabel,
            url.entityGroupLabel,
            url.filterSearchParam,
            url.groups,
            url.linkAccess,
            url.mode,
            url.modelRoles,
            url.prefersDark,
            url.preserveEntityFolderContentRole,
            url.theme,
            url.uiSettings,
            url.userAttributes,
        ]
        blob = "\n".join([i for i in blob_items if i is not None])
        hmac_hash = hmac.new(
            self.embed_secret.encode("utf-8"), blob.encode("utf-8"), hashlib.sha256
        ).digest()
        url.signature = base64.urlsafe_b64encode(hmac_hash).decode("utf-8")


@dataclass
class OmniFilterDefinition:
    """Defines an Omni dashboard filter. Used to populate an OmniFilterSet and generate the filter search params
    for an embedded dashboard URL.

    Args:
        field: Name of the Omni field a filter exists for. Generally a dot-path representing a dimension in a view.
        type: Type of the value to be filtered on.
        operator: Type of filter operation to perform.

    Attributes:
        field: Name of the Omni field a filter exists for. Generally a dot-path representing a dimension in a view.
        type: Type of the value to be filtered on.
        operator: Type of filter operation to perform.

    """

    class Type(Enum):
        """Omni Filter Type Options

        Attributes:
            number: number
            string: string
        """

        number = "number"
        string = "string"

    class Operator(Enum):
        """Omni Filter Operator Options

        Attributes:
            equals: EQUALS
            less_than: LESS_THAN
            greater_than: GREATER_THAN
            less_than_or_equal = LESS_THAN_OR_EQUAL
            greater_than_or_equal = GREATER_THAN_OR_EQUAL
            contains = CONTAINS
            between = BETWEEN
            starts_with = STARTS_WITH
            ends_with = ENDS_WITH
        """

        equals = "EQUALS"
        less_than = "LESS_THAN"
        greater_than = "GREATER_THAN"
        less_than_or_equal = "LESS_THAN_OR_EQUAL"
        greater_than_or_equal = "GREATER_THAN_OR_EQUAL"
        contains = "CONTAINS"
        between = "BETWEEN"
        starts_with = "STARTS_WITH"
        ends_with = "ENDS_WITH"

    field: str
    type: Type
    operator: Operator = Operator.equals
    is_negative: bool = False

    def get_filter_search_param_info(
        self, values: str | int | float | list[str | int | float]
    ) -> tuple[str, list[str]]:
        """Returns the key and value to be used in a query string for an Omni Dashboard.

        Args:
            values: Value or list of values to filter on.

        Returns:
            : Key and value to use in the filter search params when building an embedded dashboard URL.
        """
        if not isinstance(values, list):
            values = [values]
        filter_key = f"f--{self.field}"

        is_inclusive = False

        operator_kind = self.operator.value

        if self.operator == self.Operator.greater_than_or_equal:
            is_inclusive = True
            operator_kind = self.Operator.greater_than.value
        elif self.operator == self.Operator.less_than_or_equal:
            is_inclusive = True
            operator_kind = self.Operator.less_than.value

        filter_value_param = {
            "is_negative": self.is_negative,
            "kind": operator_kind,
            "type": self.type.value,
            "values": values,
        }

        if self.type == self.Type.number:
            filter_value_param["is_inclusive"] = is_inclusive

        filter_value = [json.dumps(filter_value_param)]
        return filter_key, filter_value


class OmniFilterSet:
    """Helper class for generating a set of filter search parameters for an embedded dashboard. This class is designed
    to abstract the complexity of the Omni filters and create a simple interface for generating the filter values to
    be used by the OmniDashboardEmbedder.

    Args:
        **filters: Arbitrary kwargs defining filter definitions. The kwarg is the name of the filter and defines the
            schema for the dict that should be passed to the get_filter_search_params method. The value for each kwarg is
            the OmniFilterDefinition object that will be used to translate the value to a viable Omni filter search param.

    """

    def __init__(self, **filters: OmniFilterDefinition) -> None:
        for value in filters.values():
            if not isinstance(value, OmniFilterDefinition):
                raise TypeError("Filters must be an OmniFilterDefinition object.")
        self._filters = filters

    @property
    def filters(self) -> dict[str, OmniFilterDefinition]:
        """Filters in this filter set. This defines the schema of the dict that should be passed to the
        `get_filter_search_params` method.
        """
        # Using a property function here to discourage manipulating filters after instantiation.
        return self._filters

    def get_filter_search_params(
        self, filter_values: dict[str, str | int | float]
    ) -> dict[str, list[str]]:
        """Given a dictionary of filter keys and values this function returns the dictionary of expected to populate
        the filter_search_params kwarg when calling OmniDashboardEmbedder.build_url. This method is ideal for
        translating query params in the encapsulating application to Omni filter search parameters.

        Args:
            filter_values: Dict where the keys are the filter names and values are the values to filter on. The
                list of available filters can be found in the `filters` property.

        Returns:
            : Dict to be passed as the `filter_search_params` kwarg in the `OmniDashboardEmbedder.build_url` method.
        """
        filter_search_params = {}
        for query_param, value in filter_values.items():
            omni_filter = self.filters[query_param]
            filter_key, filter_value = omni_filter.get_filter_search_param_info(value)
            filter_search_params[filter_key] = filter_value
        return filter_search_params



from .client import OmniApiClient
from .embed import OmniDashboardEmbedder, OmniFilterDefinition, OmniFilterSet

__version__ = "0.3.0-alpha"

__all__ = [
    "OmniApiClient",
    "OmniDashboardEmbedder",
    "OmniFilterDefinition",
    "OmniFilterSet",
]



from typing import Any

import pytest

from omni import OmniDashboardEmbedder
from omni.config import OmniConfigError
from omni.embed import OmniFilterDefinition, OmniFilterSet


@pytest.fixture
def embedder() -> OmniDashboardEmbedder:
    return OmniDashboardEmbedder(organization_name="acme", embed_secret="super_secret")


@pytest.fixture
def vanity_domain_embedder() -> OmniDashboardEmbedder:
    return OmniDashboardEmbedder(
        vanity_domain="foo.example.com", embed_secret="super_secret"
    )


@pytest.fixture(autouse=True)
def patch_uuid(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr("uuid.UUID.hex", "365f7003aa5b4f3586d9b81b4a5d9f69")


class TestUnit:
    def test_basic_url(
        self,
        embedder: OmniDashboardEmbedder,
        vanity_domain_embedder: OmniDashboardEmbedder,
    ) -> None:
        url = embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
        )
        assert (
            url
            == "https://acme.embed-omniapp.co/embed/login?contentPath=%2Fdashboards%2Fda24491e&externalId=1&name=Somebody&nonce=365f7003aa5b4f3586d9b81b4a5d9f69&signature=mToqUfdkmVSyDIGAl6Ggs9uAmGQAH9OzbbCZ-xgEU8c%3D"
        )

        url = vanity_domain_embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
        )
        assert (
            url
            == "https://foo.example.com/embed/login?contentPath=%2Fdashboards%2Fda24491e&externalId=1&name=Somebody&nonce=365f7003aa5b4f3586d9b81b4a5d9f69&signature=8HSOH-lXN3pJJ3FiaAw1XFhLCdzL44RtFS7z9S8thug%3D"
        )

    def test_kitchen_sink(
        self,
        embedder: OmniDashboardEmbedder,
        vanity_domain_embedder: OmniDashboardEmbedder,
    ) -> None:
        url = embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
            access_boost=True,
            connection_roles={"123456789": "VIEWER"},
            custom_theme={"dashboard-background": "#00FF00"},
            custom_theme_id="theme-123",
            email="user@example.com",
            entity="Acme",
            entity_folder_content_role=OmniDashboardEmbedder.ContentRole.editor,
            entity_folder_group_content_role=OmniDashboardEmbedder.ContentRole.viewer,
            entity_folder_label="EntityLabel",
            entity_group_label="GroupLabel",
            filter_search_params={"state": "GA"},
            groups=["group1", "group2"],
            link_access=True,
            mode=OmniDashboardEmbedder.AccessMode.application,
            model_roles={"model": "read"},
            prefers_dark=OmniDashboardEmbedder.PrefersDark.yes,
            preserve_entity_folder_content_role=True,
            theme=OmniDashboardEmbedder.Theme.dawn,
            ui_settings={"showNavigation": False},
            user_attributes={"country": "USA"},
        )

        assert url == (
            "https://acme.embed-omniapp.co/embed/login?"
            "contentPath=%2Fdashboards%2Fda24491e"
            "&externalId=1"
            "&name=Somebody"
            "&nonce=365f7003aa5b4f3586d9b81b4a5d9f69"
            "&accessBoost=true&connectionRoles=%7B%22123456789%22%3A%22VIEWER%22%7D"
            "&customTheme=%7B%22dashboard-background%22%3A%22%2300FF00%22%7D"
            "&customThemeId=theme-123"
            "&email=user%40example.com"
            "&entity=Acme&entityFolderContentRole=EDITOR"
            "&entityFolderGroupContentRole=VIEWER"
            "&entityFolderLabel=EntityLabel"
            "&entityGroupLabel=GroupLabel"
            "&filterSearchParam=state%3DGA"
            "&groups=%5B%22group1%22%2C%22group2%22%5D"
            "&linkAccess=__omni_link_access_open"
            "&mode=APPLICATION&modelRoles=%7B%22model%22%3A%22read%22%7D"
            "&prefersDark=true"
            "&preserveEntityFolderContentRole=true"
            "&theme=dawn"
            "&uiSettings=%7B%22showNavigation%22%3Afalse%7D"
            "&userAttributes=%7B%22country%22%3A%22USA%22%7D"
            "&signature=MmUEWVKKyuO3iDDtZGERxG1ousOQ5Ln342C-bftoxHs%3D"
        )

        url = vanity_domain_embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
            access_boost=True,
            connection_roles={"123456789": "VIEWER"},
            custom_theme={"dashboard-background": "#00FF00"},
            custom_theme_id="theme-123",
            email="user@example.com",
            entity="Acme",
            entity_folder_content_role=OmniDashboardEmbedder.ContentRole.editor,
            entity_folder_group_content_role=OmniDashboardEmbedder.ContentRole.viewer,
            entity_folder_label="EntityLabel",
            entity_group_label="GroupLabel",
            filter_search_params={"state": "GA"},
            groups=["group1", "group2"],
            link_access=True,
            mode=OmniDashboardEmbedder.AccessMode.application,
            model_roles={"model": "read"},
            prefers_dark=OmniDashboardEmbedder.PrefersDark.yes,
            preserve_entity_folder_content_role=True,
            theme=OmniDashboardEmbedder.Theme.dawn,
            ui_settings={"showNavigation": False},
            user_attributes={"country": "USA"},
        )

        assert url == (
            "https://foo.example.com/embed/login?"
            "contentPath=%2Fdashboards%2Fda24491e"
            "&externalId=1"
            "&name=Somebody"
            "&nonce=365f7003aa5b4f3586d9b81b4a5d9f69"
            "&accessBoost=true&connectionRoles=%7B%22123456789%22%3A%22VIEWER%22%7D"
            "&customTheme=%7B%22dashboard-background%22%3A%22%2300FF00%22%7D"
            "&customThemeId=theme-123"
            "&email=user%40example.com"
            "&entity=Acme&entityFolderContentRole=EDITOR"
            "&entityFolderGroupContentRole=VIEWER"
            "&entityFolderLabel=EntityLabel"
            "&entityGroupLabel=GroupLabel"
            "&filterSearchParam=state%3DGA"
            "&groups=%5B%22group1%22%2C%22group2%22%5D"
            "&linkAccess=__omni_link_access_open"
            "&mode=APPLICATION&modelRoles=%7B%22model%22%3A%22read%22%7D"
            "&prefersDark=true"
            "&preserveEntityFolderContentRole=true"
            "&theme=dawn"
            "&uiSettings=%7B%22showNavigation%22%3Afalse%7D"
            "&userAttributes=%7B%22country%22%3A%22USA%22%7D"
            "&signature=Zgjny-NQyWKZoosKJogKv-1fxVj7bmnSjOxb2O_ISz0%3D"
        )

    def test_link_access(
        self,
        embedder: OmniDashboardEmbedder,
        vanity_domain_embedder: OmniDashboardEmbedder,
    ) -> None:
        url = embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
            link_access=True,
        )
        assert (
            url
            == "https://acme.embed-omniapp.co/embed/login?contentPath=%2Fdashboards%2Fda24491e&externalId=1&name=Somebody&nonce=365f7003aa5b4f3586d9b81b4a5d9f69&linkAccess=__omni_link_access_open&signature=0-XD5eTE8myI7n2Taew-ADXmm3c_kYNCkQFHFkfOvqU%3D"
        )
        url = embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
            link_access=["abcd1234", "efgh5678"],
        )
        assert (
            url
            == "https://acme.embed-omniapp.co/embed/login?contentPath=%2Fdashboards%2Fda24491e&externalId=1&name=Somebody&nonce=365f7003aa5b4f3586d9b81b4a5d9f69&linkAccess=abcd1234%2Cefgh5678&signature=rKpBYpOKIVQmCXNfhB7J8Z0WYnlELI5KmH4uPHc1048%3D"
        )

        url = vanity_domain_embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
            link_access=True,
        )
        assert (
            url
            == "https://foo.example.com/embed/login?contentPath=%2Fdashboards%2Fda24491e&externalId=1&name=Somebody&nonce=365f7003aa5b4f3586d9b81b4a5d9f69&linkAccess=__omni_link_access_open&signature=7tFx5JlpXvJCW0llSTTioLWg53m5iGQCKKn1Yz9EO2o%3D"
        )
        url = vanity_domain_embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
            link_access=["abcd1234", "efgh5678"],
        )
        assert (
            url
            == "https://foo.example.com/embed/login?contentPath=%2Fdashboards%2Fda24491e&externalId=1&name=Somebody&nonce=365f7003aa5b4f3586d9b81b4a5d9f69&linkAccess=abcd1234%2Cefgh5678&signature=UycR_auXAIHGVTDPahgMSt4NOUxDEVc92Y3ollHcU5Q%3D"
        )

    def test_filter_search_params(self, embedder: OmniDashboardEmbedder) -> None:
        str_url = embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
            filter_search_params="state=GA&county=Fulton",
        )
        dict_url = embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
            filter_search_params={"state": "GA", "county": "Fulton"},
        )
        assert (
            str_url
            == dict_url
            == "https://acme.embed-omniapp.co/embed/login?contentPath=%2Fdashboards%2Fda24491e&externalId=1&name=Somebody&nonce=365f7003aa5b4f3586d9b81b4a5d9f69&filterSearchParam=state%3DGA%26county%3DFulton&signature=ekzTS_BikwetRpIJK40t2V11YxKPMQ_YgaiN9b9GE9Y%3D"
        )

        empty_dict_url = embedder.build_url(
            content_path="/dashboards/da24491e",
            external_id="1",
            name="Somebody",
            filter_search_params={},
        )
        assert (
            empty_dict_url
            == "https://acme.embed-omniapp.co/embed/login?contentPath=%2Fdashboards%2Fda24491e&externalId=1&name=Somebody&nonce=365f7003aa5b4f3586d9b81b4a5d9f69&signature=mToqUfdkmVSyDIGAl6Ggs9uAmGQAH9OzbbCZ-xgEU8c%3D"
        )

    def test_missing_organization_name_or_vanity_domain(self) -> None:
        with pytest.raises(OmniConfigError):
            OmniDashboardEmbedder(embed_secret="super_secret")

    def test_missing_embed_secret(self) -> None:
        with pytest.raises(OmniConfigError):
            OmniDashboardEmbedder(organization_name="acme")

        with pytest.raises(OmniConfigError):
            OmniDashboardEmbedder(vanity_domain="foo.example.com")

    def test_env_configuration_with_organization(
        self, monkeypatch: pytest.MonkeyPatch
    ) -> None:
        monkeypatch.setenv("OMNI_ORGANIZATION_NAME", "acme")
        monkeypatch.setenv("OMNI_EMBED_SECRET", "super_secret")
        embedder = OmniDashboardEmbedder()
        assert embedder.embed_secret == "super_secret"
        assert embedder.embed_login_url == "https://acme.embed-omniapp.co/embed/login"

    def test_env_configuration_with_vanity_domain(
        self, monkeypatch: pytest.MonkeyPatch
    ) -> None:
        monkeypatch.setenv("OMNI_VANITY_DOMAIN", "foo.example.com")
        monkeypatch.setenv("OMNI_EMBED_SECRET", "super_secret")
        embedder = OmniDashboardEmbedder()
        assert embedder.embed_login_url == "https://foo.example.com/embed/login"


class TestFilters:

    @pytest.mark.parametrize(
        "filter_type,operator,is_negative,values,expected",
        [
            (
                OmniFilterDefinition.Type.number,
                OmniFilterDefinition.Operator.equals,
                False,
                10,
                (
                    "f--some.attr",
                    [
                        '{"is_negative": false, "kind": "EQUALS", "type": "number", "values": [10], "is_inclusive": false}'
                    ],
                ),
            ),
            (
                OmniFilterDefinition.Type.number,
                OmniFilterDefinition.Operator.greater_than,
                False,
                10,
                (
                    "f--some.attr",
                    [
                        '{"is_negative": false, "kind": "GREATER_THAN", "type": "number", "values": [10], "is_inclusive": false}'
                    ],
                ),
            ),
            (
                OmniFilterDefinition.Type.number,
                OmniFilterDefinition.Operator.less_than,
                False,
                10,
                (
                    "f--some.attr",
                    [
                        '{"is_negative": false, "kind": "LESS_THAN", "type": "number", "values": [10], "is_inclusive": false}'
                    ],
                ),
            ),
            (
                OmniFilterDefinition.Type.number,
                OmniFilterDefinition.Operator.less_than_or_equal,
                False,
                10,
                (
                    "f--some.attr",
                    [
                        '{"is_negative": false, "kind": "LESS_THAN", "type": "number", "values": [10], "is_inclusive": true}'
                    ],
                ),
            ),
            (
                OmniFilterDefinition.Type.number,
                OmniFilterDefinition.Operator.greater_than_or_equal,
                False,
                10,
                (
                    "f--some.attr",
                    [
                        '{"is_negative": false, "kind": "GREATER_THAN", "type": "number", "values": [10], "is_inclusive": true}'
                    ],
                ),
            ),
            (
                OmniFilterDefinition.Type.number,
                OmniFilterDefinition.Operator.between,
                False,
                [10, 25],
                (
                    "f--some.attr",
                    [
                        '{"is_negative": false, "kind": "BETWEEN", "type": "number", "values": [10, 25], "is_inclusive": false}'
                    ],
                ),
            ),
            (
                OmniFilterDefinition.Type.string,
                OmniFilterDefinition.Operator.equals,
                False,
                "California",
                (
                    "f--some.attr",
                    [
                        '{"is_negative": false, "kind": "EQUALS", "type": "string", "values": ["California"]}'
                    ],
                ),
            ),
            (
                OmniFilterDefinition.Type.string,
                OmniFilterDefinition.Operator.starts_with,
                False,
                "California",
                (
                    "f--some.attr",
                    [
                        '{"is_negative": false, "kind": "STARTS_WITH", "type": "string", "values": ["California"]}'
                    ],
                ),
            ),
            (
                OmniFilterDefinition.Type.string,
                OmniFilterDefinition.Operator.ends_with,
                True,
                "California",
                (
                    "f--some.attr",
                    [
                        '{"is_negative": true, "kind": "ENDS_WITH", "type": "string", "values": ["California"]}'
                    ],
                ),
            ),
            (
                OmniFilterDefinition.Type.string,
                OmniFilterDefinition.Operator.contains,
                True,
                "California",
                (
                    "f--some.attr",
                    [
                        '{"is_negative": true, "kind": "CONTAINS", "type": "string", "values": ["California"]}'
                    ],
                ),
            ),
        ],
    )
    def test_filters(
        self,
        filter_type: OmniFilterDefinition.Type,
        operator: OmniFilterDefinition.Operator,
        is_negative: bool,
        values: Any,
        expected: Any,
    ) -> None:
        filter = OmniFilterDefinition(
            field="some.attr",
            type=filter_type,
            operator=operator,
            is_negative=is_negative,
        )
        assert filter.get_filter_search_param_info(values) == expected

    def test_filter_negative_not_required(self) -> None:
        filter = OmniFilterDefinition(
            field="some.attr",
            type=OmniFilterDefinition.Type.string,
            operator=OmniFilterDefinition.Operator.contains,
        )
        assert filter.get_filter_search_param_info("California") == (
            "f--some.attr",
            [
                '{"is_negative": false, "kind": "CONTAINS", "type": "string", "values": ["California"]}'
            ],
        )

    def test_bad_filters_in_filter_set(self) -> None:
        with pytest.raises(TypeError):
            OmniFilterSet(fail="nope")

    def test_filter_set(self) -> None:
        filter_set = OmniFilterSet(
            latitude=OmniFilterDefinition(
                field="address.latitude_filter",
                type=OmniFilterDefinition.Type.number,
            ),
            longitude=OmniFilterDefinition(
                field="address.longitude_filter",
                type=OmniFilterDefinition.Type.number,
            ),
            distance=OmniFilterDefinition(
                field="address.distance_selected_to_address_in_miles",
                type=OmniFilterDefinition.Type.number,
                operator=OmniFilterDefinition.Operator.less_than,
            ),
        )
        assert filter_set.get_filter_search_params(
            {"latitude": 33.555, "longitude": -117.602, "distance": 10}
        ) == {
            "f--address.distance_selected_to_address_in_miles": [
                '{"is_negative": false, "kind": "LESS_THAN", "type": "number", "values": [10], "is_inclusive": false}'
            ],
            "f--address.latitude_filter": [
                '{"is_negative": false, "kind": "EQUALS", "type": "number", "values": [33.555], "is_inclusive": false}'
            ],
            "f--address.longitude_filter": [
                '{"is_negative": false, "kind": "EQUALS", "type": "number", "values": [-117.602], "is_inclusive": false}'
            ],
        }



import json


def compact_json_dump(data: dict | list) -> str:
    """Dumps a dictionary to a JSON string with no extra whitespace."""
    return json.dumps(
        data,
        sort_keys=True,
        separators=(",", ":"),
    )



from __future__ import annotations

import os
from dataclasses import dataclass
from typing import Any


@dataclass
class OmniConfig:
    """Fetches configuration options from arguments or environment variables."""

    required_attrs: list[str]
    organization_name: str | None = None
    vanity_domain: str | None = None
    embed_secret: str | None = None
    api_key: str | None = None

    @staticmethod
    def _get_env_var(attr_name: str) -> str:
        return f"OMNI_{attr_name.upper()}"

    def __getattribute__(self, name: str) -> Any:
        """Overrides built-in `__getattribute__` to fall back to environment variables with the OMNI_ prefix."""
        value = super().__getattribute__(name)
        if value is None:
            value = os.environ.get(self._get_env_var(name))
        return value

    def __post_init__(self) -> None:
        """Raises an error if a required config setting is missing."""
        if missing_required_attrs := [
            a for a in self.required_attrs if not getattr(self, a)
        ]:
            message = (
                f"Omni SDK has not been configured correctly. You must pass the arguments {missing_required_attrs} and/or set "
                f"the environment variables {[self._get_env_var(a) for a in missing_required_attrs]}. Please see the documentation "
                "for more information on configuration."
            )
            raise OmniConfigError(message)


class OmniConfigError(Exception):
    pass



from __future__ import annotations

from typing import Literal

import requests

from .config import OmniConfig


class OmniApiClient:
    """Class for interacting with the Omni REST API. There are low level functions for making direct requests to the
    API (get, post, put, delete). These methods take a "path" arg that is equivalent to the path given in the Omni
    API docs. The client also includes convenience methods for common tasks.

    Args:
        organization_name: Omni organization name. OMNI_ORGANIZATION_NAME environment variable will be used as a fallback.
        api_key: Omni API key. OMNI_API_KEY environment variable will be used as a fallback.

    Attributes:
        base_url: Omni REST API base URL that paths will be appended to.
        api_key: Omni API key.
    """

    def __init__(
        self, organization_name: str | None = None, api_key: str | None = None
    ) -> None:
        omni_config = OmniConfig(
            required_attrs=["organization_name", "api_key"],
            organization_name=organization_name,
            api_key=api_key,
        )
        self.base_url = f"https://{omni_config.organization_name}.omniapp.co/api"
        self.api_key = omni_config.api_key

    def refresh_model(self, model_id: str) -> bool:
        """Refreshes this model to reflect the latest structures (schemas, views, fields) from the data source.
        This will remove any structures that are no longer present in the source, but will not remove anything
        created by users.

        Args:
            model_id (str): The ID of the Omni model to refresh.

        Returns:
            : True if successful.
        """
        self.post(f"/v0/model/{model_id}/refresh")
        return True

    def get(self, path: str, params: dict | None = None) -> dict:
        """Makes a GET request to the Omni REST API.

        Args:
            path: The path in the Omni REST API to make a GET request.
            params: Query string parameters to use in the GET request.

        Returns:
            JSON response from the Omni REST API.
        """
        return self._request("GET", path, params=params)

    def post(self, path: str, json_data: dict | None = None) -> dict:
        """Makes a POST request to the Omni REST API.

        Args:
            path: The path in the Omni REST API to make a POST request.
            json_data: Query string parameters to use in the POST request.

        Returns:
            JSON response from the Omni REST API.
        """
        return self._request("POST", path, json_data=json_data)

    def put(self, path: str, json_data: dict | None = None) -> dict:
        """Makes a PUT request to the Omni REST API.

        Args:
            path: The path in the Omni REST API to make a PUT request.
            json_data: Query string parameters to use in the PUT request.

        Returns:
            JSON response from the Omni REST API.
        """
        return self._request("PUT", path, json_data=json_data)

    def delete(self, path: str) -> dict:
        """Makes a DELETE request to the Omni REST API.

        Returns:
            JSON response from the Omni REST API.
        """
        return self._request("DELETE", path)

    def _get_url(self, path: str) -> str:
        return f"{self.base_url.strip('/')}/{path.strip('/')}"

    def _request(
        self,
        method: Literal["GET", "POST", "PUT", "DELETE"],
        path: str,
        json_data: dict | None = None,
        params: dict | None = None,
    ) -> dict:
        response = requests.request(
            method=method,
            headers={"Authorization": f"Bearer {self.api_key}"},
            url=self._get_url(path),
            json=json_data,
            params=params,
        )
        response.raise_for_status()
        return response.json()
